package it.unitn.disi.prog2.esameluglio18;

import it.unitn.disi.prog2.esameluglio18.figures.Circle;
import it.unitn.disi.prog2.esameluglio18.figures.Empty;
import it.unitn.disi.prog2.esameluglio18.figures.Figura;
import it.unitn.disi.prog2.esameluglio18.figures.Hexagon;
import it.unitn.disi.prog2.esameluglio18.figures.Triangle;
import javafx.scene.layout.GridPane;
/**
 * classe che implementa la colonna di sinistra,
 * e che comprende imetodiagenti su di essa
 * @author ronchet
 */
public class Stack extends GridPane {
    final int NITEMS=5;
    Figura items[]=new Figura[NITEMS];
    Class nextFiguraType=null;
    String nextFiguraTypeString=null;
    int marker=0; //indicatore della prima cella vuota
    Stack(){
        marker=0; 
        for (int i=0; i<NITEMS; i++) {
            this.add((new Empty()).getNode(), 0, i);
        }
    }
    /**
     * implementazione "elegante" che fa uso della classe Class per rappresentare
     * lo "stampino" con il quale produrre le figure richieste
     * @param f Classe di cui creare nuove istanze quando richiesto
     */
    void setNextFiguraType(Class<? extends Figura> f){
        nextFiguraType=f;
    }
    /**
     * implementazione più semplice ma meno elegante - fa uso di stringhe
     * e si appoggia poi su uno switch - vedi il metodo addFigura sotto.
     * Analogamente potrebbe essere implementato con un Enum.
     */
    void setNextFiguraType(String f){
        nextFiguraTypeString=f;
    }
    /**
     * metodo che incapsula la stampa in console e la stampa in FinestraMessaggi -
     * commentando opportunamente una delle due scelte si avrà solo l'altra
     * @param s messaggio da stampare
     */
    void print(String s){
        /* implementazione stampa in console */
        System.out.println(s);
        /* implementazione stampa in finestra */
        new FinestraMessaggi(s);
    }
    /**
     * metodo per aggiungere la figura attualmente selezionata
     * Nota: vi sono due implementazioni: quella basata su Class
     * e quella basata su string. Va decommentata quella che si vole usare,
     * e commentata l'altra.
     * @return true se è riuscito a generare la figura, false altrimenti
     */
    boolean addFigura(){
        Figura f=null;
        /* implementazione che si appoggia su nextFiguraType */
        try {
            f = (Figura)(nextFiguraType.newInstance());
        } catch (InstantiationException | IllegalAccessException | NullPointerException ex) {
            print(ex.toString()+" in Stack.addFigura()");
            return false;
        }
        /* implementazione che si appoggia su nextFiguraTypeString */
        /* 
        switch (nextFiguraTypeString) {
            case "Triangolo": f=new Triangle(); break;
            case "Cerchio": f=new Circle(); break;
            case "Esagono": f=new Hexagon(); break;   
        }
        */
        if (marker>=NITEMS) { // stack pieno
            print("Stack pieno");
            return false;
        } 
        //items.add(f);
        items[marker]=f;
        this.add(f.getNode(),0,NITEMS-marker-1);
        marker++;
        return true;
    }
    /**
     * metodo per rimuovere l'elemento in cima alla colonna
     * @return true se è riuscito, false altrimenti
     */
    boolean removeTop(){
        if (marker<=0){
            print("Stack vuoto");
            return false;       
        }
        this.getChildren().remove(items[marker-1].getNode());
        marker--;
        items[marker]=null;
        return true;
    }
    /**
     * metodo per rimuovere l'elemento in fondo alla colonna
     * @return true se è riuscito, false altrimenti
     */    
    boolean removeBottom(){
        if (marker<=0){
            print("Stack vuoto");
            return false;       
        }
        this.getChildren().remove(items[0].getNode());
        for (int i=0;i<marker-1;i++) {
            this.getChildren().remove(items[i+1].getNode());
            items[i]=items[i+1];
            this.add(items[i].getNode(),0,NITEMS-i-1);
        }
        marker--;
        items[marker]=null;
        return true;
    }
    /**
     * metodo che stampa il contenuto della colonna
     */
    void printContent(){
        String s="=================\n";
        for (int i=NITEMS-1;i>=0;i--) {
            s=s+items[i]+"\n";
        }
        s=s+"=================";
        print(s);
        
    }
}
